@@ -5,6 +5,9 @@ use Mojo::UserAgent;
 use Mojo::Netdata::Util qw(logf safe_id);
 use Time::HiRes qw(time);
 
+require Mojo::Netdata;
+our $VERSION = $Mojo::Netdata::VERSION;
+
 has jobs => sub ($self) { +[] };
 has type => 'HTTP';
 has ua   => sub { Mojo::UserAgent->new(insecure => 0, connect_timeout => 5, request_timeout => 5) };
@@ -19,6 +22,7 @@ sub register ($self, $config, $netdata) {
   $self->ua->connect_timeout($config->{connect_timeout}) if defined $config->{connect_timeout};
   $self->ua->request_timeout($config->{request_timeout}) if defined $config->{request_timeout};
   $self->ua->proxy->detect                               if $config->{proxy} // 1;
+  $self->ua->transactor->name($config->{user_agent} || "Mojo-Netdata/$VERSION (Perl)");
   $self->jobs([]);
 
   my @jobs = ref $config->{jobs} eq 'HASH' ? %{$config->{jobs}} : @{$config->{jobs}};
@@ -37,12 +41,9 @@ sub update_p ($self) {
   for my $job (@{$self->jobs}) {
     my $tx = $ua->build_tx(@{$job->[0]});
     push @p, $ua->start_p($tx)->then(sub ($tx) {
-      my $t1 = time;
-      logf(debug => '%s %s == %s', $tx->req->method, $tx->req->url, $tx->res->code);
       $job->[1]->($tx, $t0);
     })->catch(sub ($err) {
-      logf(warnings => '%s %s == %s', $tx->req->method, $tx->req->url, $err);
-      $job->[1]->($tx, $t0);
+      $job->[1]->($tx, $t0, {message => $err});
     });
   }
 
@@ -55,6 +56,7 @@ sub _make_job ($self, $url, $params, $defaults) {
 
   my $headers = Mojo::Headers->new->from_hash($defaults->{headers} || {});
   $headers->header($_ => $params->{headers}{$_}) for keys %{$params->{headers} || {}};
+  ($headers->header(Host => $url->host), $url->host($params->{via})) if $params->{via};
 
   my $dimension = $params->{dimension} || $headers->host || $url->host;
   my $family    = $params->{family}    || $defaults->{family} || $headers->host || $url->host;
@@ -73,9 +75,15 @@ sub _make_job ($self, $url, $params, $defaults) {
   $code_chart->dimension($dimension => {});
   $time_chart->dimension($dimension => {});
 
-  my $update = sub ($tx, $t0) {
+  my $update = sub ($tx, $t0, $err = undef) {
+    $err ||= $tx->error;
+    my $req  = $tx->req;
+    my $code = $tx->res->code // 0;
+    my @msg  = ($req->method, $req->url, $err || {code => $code}, $req->headers->to_hash(1));
+    logf(($err ? 'warnings' : 'debug'), '%s %s == %s %s', @msg);
+
     $time_chart->dimension($dimension => {value => int(1000 * (time - $t0))});
-    $code_chart->dimension($dimension => {value => $tx->res->code // 0});
+    $code_chart->dimension($dimension => {value => $code});
   };
 
   my @data;
@@ -86,7 +94,6 @@ sub _make_job ($self, $url, $params, $defaults) {
     : exists $params->{body} ? ($params->{body})
     :                          ();
 
-  logf(info => 'Tracking %s', $url);
   return [[$params->{method} || 'GET', $url->to_unsafe_string, @data], $update];
 }
 
@@ -111,11 +118,12 @@ as it has the C<.conf.pl> extension.
     collector => 'Mojo::Netdata::Collector::HTTP',
 
     # Optional
-    insecure        => 0,  # Set to "1" to allow insecure SSL/TLS connections
-    connect_timeout => 5,  # Max time for the connection to be established
-    request_timeout => 5,  # Max time for the whole request to complete
-    proxy           => 1,  # Set to "0" to disable proxy auto-detect
-    update_every    => 30, # How often to run the "jobs" below
+    insecure        => 0,     # Set to "1" to allow insecure SSL/TLS connections
+    connect_timeout => 5,     # Max time for the connection to be established
+    request_timeout => 5,     # Max time for the whole request to complete
+    proxy           => 1,     # Set to "0" to disable proxy auto-detect
+    update_every    => 30,    # How often to run the "jobs" below
+    user_agent      => '...', # Custom User-Agent name
 
     # Default values, unless defined in the job
     family  => 'default-family-name',
@@ -130,10 +138,12 @@ as it has the C<.conf.pl> extension.
 
       # URL and config parameters
       'https://example.com' => {
-
-        method => 'GET',               # GET (Default), HEAD, POST, ...
+        method  => 'GET',              # GET (Default), HEAD, POST, ...
         headers => {'X-Foo' => 'bar'}, # HTTP headers
 
+        # Replace "host" in the URL with this IP and set the "Host" header
+        via => '192.168.2.1',
+
         # Set "dimension" to get a custom label in the chart.
         # Default to the "Host" header or the host part of the URL.
         dimension => 'foo', # Default: "example.com"
